/***************************************************************************
 *   Copyright (C) 2007 by David Rivoal                                    *
 *   davidrivoal@free.fr                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "mailbox.h"
#include "mailbox.moc"

Mailbox::Mailbox(QString f_boxserver, QString f_user, QString f_password, bool f_boxIsPop)
{
	boxserver = f_boxserver;
	user = f_user;
	password = f_password;
	boxIsPop = f_boxIsPop;

	timerTimeout = new QTimer(this);
	connect(timerTimeout, SIGNAL(timeout()), this, SLOT(socketTimeout()));
}

/********************************************************************/

Mailbox::~Mailbox()
{
	delete(boxSocket);
	delete(socketBuf);
	delete(serverLookup);
	delete(timerTimeout);
}

/********************************************************************/

void Mailbox::changeBox(QString f_boxserver, QString f_user, QString f_password, bool f_boxIsPop)
{
	boxserver = f_boxserver;
	user = f_user;
	password = f_password;
	boxIsPop = f_boxIsPop;
}

/********************************************************************/

bool Mailbox::isOk(QString resp)
{
	if(resp.find("OK") == -1)
		return false;

	return true;
}

/********************************************************************/

void Mailbox::requestServer()
{
	QString buf;

	// Create the socket
	boxSocket = new QSocket(this);
	// Signal sent each time it's possible to read something
	connect(boxSocket, SIGNAL(readyRead()), this, SLOT(socketRead()));

	// Connection to server with hostname resolution
	serverLookup = new QDns(boxserver, QDns::A);
	connect(serverLookup, SIGNAL(resultsReady()), this, SLOT(connectServer()));

	state = S_INIT;
}

/********************************************************************/

void Mailbox::connectServer()
{
	// Get the IP address of the server
	QValueList<QHostAddress> listServers = serverLookup->addresses();
	if(listServers.isEmpty())
	{
		if(!serverLookup->isWorking())
			emit serverResponse(1);

		return;
	}

	// Connect the socket to the server and launch timeout
	timerTimeout->start(sockTimeout, true);
	if(boxIsPop)
		boxSocket->connectToHost(listServers.first().toString(), 110);
	else
		boxSocket->connectToHost(listServers.first().toString(), 143);

	// Buffer to send instructions to the server
	socketBuf = new QTextStream(boxSocket);
}


/********************************************************************/

void Mailbox::socketRead()
{
	if(boxIsPop)
		popRead();
	else
		imapRead();
}

/********************************************************************/

void Mailbox::popRead()
{
	int nbMails;

	if (!boxSocket->canReadLine())
		return;

	QString responseLine;

	if(state == S_INIT)
	{
		responseLine = boxSocket->readLine();
		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(2);
			return;
		}

		// Connection is Ok (previous test), we send the USER command
		*socketBuf << "USER " << user << "\n";

		state = S_USER;
	}
	else if(state == S_USER)
	{
		responseLine = boxSocket->readLine();

		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(3);
			return;
		}

		// We send the PASS command
		*socketBuf << "PASS " << password << "\n";

		state = S_PASS;
	}
	else if(state == S_PASS)
	{
		responseLine = boxSocket->readLine();

		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(3);
			return;
		}

		// Authentication Ok, we ask for STAT
		*socketBuf << "STAT\n";

		state = S_STAT;
	}
	else if(state == S_STAT)
	{
		responseLine = boxSocket->readLine();

		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(4);
			return;
		}

		// Finally get the f**king number
		nbMails = responseLine.section(0x20, 1, 1).toInt();

		*socketBuf << "QUIT\n";

		state = S_QUIT;

		// Everything went OK, we stop the timeout timer and send the response
		timerTimeout->stop();
		emit serverResponse(0, nbMails);
	}
}

/********************************************************************/

void Mailbox::imapRead()
{
	int nbMails;

	if (!boxSocket->canReadLine())
		return;

	QString responseLine;

	if(state == S_INIT)
	{
		responseLine = boxSocket->readLine();
		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(2);
			return;
		}

		// Connection is Ok (previous test), we send the USER command
		*socketBuf << "a1 login " << user << " " << password << "\n";

		state = S_AUTH;
	}
	else if(state == S_AUTH)
	{
		responseLine = boxSocket->readLine();

		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(3);
			return;
		}

		// Authentication Ok, we ask for STAT
		*socketBuf << "a2 examine inbox\n";

		state = S_STAT;
	}
	else if(state == S_STAT)
	{
		while(boxSocket->canReadLine())
		{
			responseLine = boxSocket->readLine();

			// number of mails here
			if(responseLine.find("EXISTS") != -1)
				nbMails = responseLine.section(0x20, 1, 1).toInt();
		}

		if(!isOk(responseLine))
		{
			timerTimeout->stop();
			emit serverResponse(4);
			return;
		}

		*socketBuf << "a3 logout\n";

		state = S_QUIT;

		// Everything went OK, we stop the timeout timer and send the response
		timerTimeout->stop();
		emit serverResponse(0, nbMails);
	}
}

/********************************************************************/

void Mailbox::socketTimeout()
{
	// Send an error message
	emit serverResponse(5);

	// Close the socket
	boxSocket->close();
}
