/***************************************************************************
 *   Copyright (C) 2007 by David Rivoal                                    *
 *   davidrivoal@free.fr                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qstringlist.h>

#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kaboutapplication.h>
#include <kconfigdialog.h>
#include <kprocess.h>
#include <kpassivepopup.h>

#include "mailkount.h"
#include "prefgenerallayout.h"

MailKount::MailKount(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
	: KPanelApplet(configFile, type, actions, parent, name)
{
	// Some init
	box1Updating = box2Updating = false;
	nbMailsBox1 = nbMailsBox2 = nbNewMailsBox1 = nbNewMailsBox2 = 0;

	// Get the config
	MailKountConfig::instance(configFile);

	// Do we have a second box?
	isBox2Enabled = MailKountConfig::secondBox();

	// Init of the mailboxes
	mailbox1 = new Mailbox(MailKountConfig::box1Address(), MailKountConfig::box1User(), MailKountConfig::box1Password(), MailKountConfig::box1Pop());
	mailbox2 = new Mailbox(MailKountConfig::box2Address(), MailKountConfig::box2User(), MailKountConfig::box2Password(), MailKountConfig::box2Pop());

	// Connect the response from mailboxes to our management functions
	connect(mailbox1, SIGNAL(serverResponse(int, int)), this, SLOT(box1Updated(int, int)));
	connect(mailbox2, SIGNAL(serverResponse(int, int)), this, SLOT(box2Updated(int, int)));

	// Our main layout
	mainLayout = new QGridLayout(this);

	// Our 4 labels
	QColor colorFontMailKount(MailKountConfig::fontColor());
	labelMailbox1Name = new QLabel(this);
	labelMailbox1Num = new QLabel("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>", this);
	labelMailbox2Name = new QLabel(this);
	labelMailbox2Num = new QLabel("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>", this);

	// We get the number of messages in our boxes
	checkNbMails(true);

	// We display the widgets
	displayAll();

	// Set the context menu
	contextMenu = new KPopupMenu();
	contextMenu->insertTitle("Mailkount");
	contextMenu->insertItem(SmallIcon("mail_find2"), i18n("&Read mails"), mReadMail);	
	contextMenu->insertItem(SmallIcon("mail_get"), i18n("&Check now"), mCheckMail);	
	contextMenu->insertSeparator();
	contextMenu->insertItem(SmallIcon("configure"), i18n("&Configure"), mConfigure);	
	contextMenu->insertItem(SmallIcon("help"), i18n("&About"), mAbout);

	// Set the timer for refresh
	timerMailKount = new QTimer(this);
	connect(timerMailKount, SIGNAL(timeout()), this, SLOT(refresh()));
	timerMailKount->start(60000 * MailKountConfig::refreshInterval());
}

/***********************************************************************************************/

MailKount::~MailKount()
{
	// save configuration
	MailKountConfig::writeConfig();

	timerMailKount->stop();
	delete(timerMailKount);

	delete(contextMenu);
	delete(labelMailbox1Name);
	delete(labelMailbox1Num);
	delete(labelMailbox2Name);
	delete(labelMailbox2Num);
	delete(mainLayout);

	delete(mailbox1);
	delete(mailbox2);
}

/***********************************************************************************************/

void MailKount::checkNbMails(bool initChk)
{
	// First check
	initCheck = initChk;

	// Update box 1
	box1Updating = true;
	mailbox1->requestServer();
	
	// Update box 2, if enabled
	if(isBox2Enabled)
	{
		box2Updating = true;
		mailbox2->requestServer();
	}
}

/***********************************************************************************************/

void MailKount::box1Updated(int errorCode, int nbMess)
{
	if(errorCode == 0)
	{
		// We have our graal ... or we should!
		nbNewMailsBox1 = nbMess - nbMailsBox1;
		nbMailsBox1 = nbMess;

		box1Updating = false;
		displayResult();
	}
	else
	{
		nbNewMailsBox1 = 0;
		errorMessage(1, errorCode);
		box1Updating = false;
	}
}

/***********************************************************************************************/

void MailKount::box2Updated(int errorCode, int nbMess)
{
	if(errorCode == 0)
	{
		// We have our graal ... or we should!
		nbNewMailsBox2 = nbMess - nbMailsBox2;
		nbMailsBox2 = nbMess;

		box2Updating = false;
		displayResult();
	}
	else
	{
		nbNewMailsBox2 = 0;
		errorMessage(2, errorCode);
		box2Updating = false;
	}
}

/***********************************************************************************************/

void MailKount::displayResult()
{
	// Be sure that the boxes are not updating now
	if(box1Updating || box2Updating)
		return;

	// Font color
	QColor colorFontMailKount(MailKountConfig::fontColor());

	if(nbMailsBox1 >= 0)
		labelMailbox1Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+srtNbMailsBox1.setNum(nbMailsBox1)+"</font>");
	else
		labelMailbox1Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>");

	if(isBox2Enabled)
	{
		if(nbMailsBox2 >= 0)
			labelMailbox2Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+srtNbMailsBox2.setNum(nbMailsBox2)+"</font>");
		else
			labelMailbox2Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>");

		// display a passive pop up, except for the first check
		if(!initCheck)
		{
			if(nbNewMailsBox1 == 0)
			{
				if(nbNewMailsBox2 == 1)
					passiveMessage(i18n("You received 1 new message in your second box."));
				else if(nbNewMailsBox2 > 1)
					passiveMessage(i18n("You received %1 new messages in your second box.").arg(nbNewMailsBox2));
			}
			else if(nbNewMailsBox1 == 1)
			{
				if(nbNewMailsBox2 == 0)
					passiveMessage(i18n("You received 1 new message in your first box."));
				else if(nbNewMailsBox2 == 1)
					passiveMessage(i18n("You received 1 new message in your two boxes."));
				else if(nbNewMailsBox2 > 1)
					passiveMessage(i18n("You received 1 new message in your first box and %1 new messages in your second box.").arg(nbNewMailsBox2));
			}
			else if(nbNewMailsBox1 > 1)
			{
				if(nbNewMailsBox2 == 0)
					passiveMessage(i18n("You received %1 new messages in your first box.").arg(nbNewMailsBox1));
				else if(nbNewMailsBox2 == 1)
					passiveMessage(i18n("You received %1 new messages in your first box and 1 new message in your second box.").arg(nbNewMailsBox1));
				else if(nbNewMailsBox2 > 1)
					passiveMessage(i18n("You received %1 new messages in your first box and %2 new messages in your second box.").arg(nbNewMailsBox1).arg(nbNewMailsBox2));
			}
		}
	}
	else if(!initCheck) // only one box
	{
		if(nbNewMailsBox1 == 1)
			passiveMessage(i18n("You received 1 new message in your box."));
		else if(nbNewMailsBox1 > 1)
			passiveMessage(i18n("You received %1 new messages in your box.").arg(nbNewMailsBox1));
	}

	update();
}

/***********************************************************************************************/

void MailKount::displayAll()
{
	// display font and color
	QFont fontMailKount(MailKountConfig::displayFont());
	QColor colorFontMailKount(MailKountConfig::fontColor());
	
	// names of the boxes
	labelMailbox1Name->setText("<font color=\""+colorFontMailKount.name()+"\">"+MailKountConfig::box1Name()+"</font>");
	labelMailbox2Name->setText("<font color=\""+colorFontMailKount.name()+"\">"+MailKountConfig::box2Name()+"</font>");

	// display the second box ?
	labelMailbox2Name->setHidden(!isBox2Enabled);
	labelMailbox2Num->setHidden(!isBox2Enabled);

	// managing the layout
	// first case : horizontal
	if(orientation() == Qt::Horizontal)
	{
		// We have two boxes
		if(isBox2Enabled)
		{
			// prepare the layout
			mainLayout->expand(2,2);

			// We finally display the two boxes
			labelMailbox1Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Name, 0, 0);
			labelMailbox1Num->setAlignment(Qt::AlignRight | Qt::AlignVCenter | Qt::ExpandTabs);
			labelMailbox1Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Num, 0, 1);
			
			labelMailbox2Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox2Name, 1, 0);
			labelMailbox2Num->setAlignment(Qt::AlignRight | Qt::AlignVCenter | Qt::ExpandTabs);
			labelMailbox2Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox2Num, 1, 1);
		}
		else // Only one box
		{
			// prepare the layout
			mainLayout->expand(1,2);

			// We display only the first box
			labelMailbox1Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Name, 0, 0);
			labelMailbox1Num->setAlignment(Qt::AlignRight | Qt::AlignVCenter | Qt::ExpandTabs);
			labelMailbox1Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Num, 0, 1);
		}
	}
	else // second case : vertical
	{
		// We have two boxes
		if(isBox2Enabled)
		{
			// prepare the layout
			mainLayout->expand(4,1);
		
			// We finally display the two boxes
			labelMailbox1Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Name, 0, 0);
			labelMailbox1Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Num, 1, 0);
			
			labelMailbox2Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox2Name, 2, 0);
			labelMailbox2Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox2Num, 3, 0);
		}
		else // Only one box
		{
			// prepare the layout
			mainLayout->expand(2,1);
		
			// We display only the first box
			labelMailbox1Name->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Name, 0, 0);
			labelMailbox1Num->setFont(fontMailKount);
			mainLayout->addWidget(labelMailbox1Num, 1, 0);
		}
	}
}

/***********************************************************************************************/

void MailKount::preferences()
{
	if(KConfigDialog::showDialog("settings"))
		return;

	KConfigDialog* dialog = new KConfigDialog(this, "settings", MailKountConfig::self(), KDialogBase::Plain, KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel);
	SettingForm* confWdg = new SettingForm(0, "settings");
	dialog->addPage(confWdg, i18n("MailKount Preferences"), "configure");

	connect(dialog, SIGNAL(settingsChanged()), this, SLOT(updateSettings()));

	dialog->show();
}

/***********************************************************************************************/

void MailKount::updateSettings()
{
	bool previousEnabled=isBox2Enabled;
	
	// save configuration
	MailKountConfig::writeConfig();

	isBox2Enabled = MailKountConfig::secondBox();

	// Did second box was enabled/disabled?
	if(previousEnabled ^ isBox2Enabled)
	{
		mainLayout->invalidate();
		delete(mainLayout);
		mainLayout = new QGridLayout(this);
	}

	// we redisplay everything
	displayAll();

	// new mailboxes
	mailbox1->changeBox(MailKountConfig::box1Address(), MailKountConfig::box1User(), MailKountConfig::box1Password(), MailKountConfig::box1Pop());
	mailbox2->changeBox(MailKountConfig::box2Address(), MailKountConfig::box2User(), MailKountConfig::box2Password(), MailKountConfig::box2Pop());

	// check mails and restart timer
	checkNbMails();
	timerMailKount->stop();
	timerMailKount->start(60000 * MailKountConfig::refreshInterval());
}

/***********************************************************************************************/

int MailKount::widthForHeight(int height) const
{
	int sizeBox1, sizeBox2;
	
	sizeBox1 = labelMailbox1Name->minimumSizeHint().width() + labelMailbox1Num->minimumSizeHint().width() + 8;
	
	if(isBox2Enabled)
	{
		sizeBox2 = labelMailbox2Name->minimumSizeHint().width() + labelMailbox2Num->minimumSizeHint().width() + 8;

		return (sizeBox1 < sizeBox2)?sizeBox2:sizeBox1;
	}
	else
	{
		return sizeBox1;
	}
}

/***********************************************************************************************/

int MailKount::heightForWidth(int width) const
{
	if(isBox2Enabled)
	{
		return 4*labelMailbox1Name->minimumSizeHint().height();
	}
	else
	{
		return 2*labelMailbox1Name->minimumSizeHint().height();
	}
}

/***********************************************************************************************/

void MailKount::mousePressEvent(QMouseEvent* e)
{
	// Be sure that the boxes are not updating now
	if(box1Updating || box2Updating)
	{
		passiveMessage(i18n("Updating box(es) now, please wait update is finished and click again."));
		return;
	}

	if(e->button() == RightButton)
	{
		switch(contextMenu->exec(mapToGlobal(e->pos())))
		{
			case mConfigure:
				preferences();
				break;

			case mAbout:
				about();
				break;

			case mReadMail:
				commandReadMails();
				break;

			case mCheckMail:
				checkNbMails();
				break;

			default:
				break;
		}
	}
}

/***********************************************************************************************/

void MailKount::commandReadMails()
{
	QStringList command = QStringList::split(0x20, MailKountConfig::viewCommand());
	KProcess *proc = new KProcess;
	*proc << command;
	proc->start();
}

/***********************************************************************************************/

void MailKount::refresh()
{
	checkNbMails();
}

/***********************************************************************************************/

void MailKount::errorMessage(int box, int errorCode)
{
	QString boite;
	QString messageErreur;
	QColor colorFontMailKount(MailKountConfig::fontColor());

	if(box == 1)
	{
		boite = i18n("primary");
		labelMailbox1Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>");
	}
	else
	{
		boite = i18n("secondary");
		labelMailbox2Num->setText("<font color=\""+colorFontMailKount.name()+"\">"+"-</font>");
	}
	update();

	switch(errorCode)
	{
		case 1 :
			messageErreur = i18n("DNS lookup error on %1 server.\nCheck your configuration.").arg(boite);
			break;

		case 2 :
			messageErreur = i18n("Connection to %1 server refused.").arg(boite);
			break;

		case 3 :
			messageErreur = i18n("User authentication to %1 server refused.").arg(boite);
			break;

		case 4 :
			messageErreur = i18n("Unable to get number of messages on %1 server.").arg(boite);
			break;

		case 5 :
			messageErreur = i18n("Connection to %1 server timeout.").arg(boite);
			break;

		default :
			messageErreur = i18n("Congratulation, you are now seeing a message for an unknown error on %1 box.\nPlease, tell me your method!").arg(boite);
			break;
	}

	passiveMessage(messageErreur);
}

/***********************************************************************************************/

void MailKount::passiveMessage(QString mess)
{
	KPassivePopup *passPopup = new KPassivePopup(this);
	passPopup->setView("~~~ MailKount ~~~", mess);
	passPopup->setTimeout(5000);
	connect(passPopup, SIGNAL(clicked()), passPopup, SLOT(hide()));
	passPopup->show();
}

/***********************************************************************************************/

void MailKount::positionChange(Position p)
{
	delete(mainLayout);
	mainLayout = new QGridLayout(this);
	
	displayAll();
}

/***********************************************************************************************/

void MailKount::about()
{
	KAboutData aboutData ("MailKount", "MailKount", VERSION,
						  I18N_NOOP("Displays the number of mails on one or two mailboxes."),
						  KAboutData::License_GPL_V2,
						  "(c) 2007, David Rivoal",
						  "",
						  "",
						  "davidrivoal@free.fr");

	aboutData.addAuthor("David Rivoal", "", "davidrivoal@free.fr");

	KAboutApplication aboutDialog(&aboutData, this, NULL, 0);

	aboutDialog.exec();
}

/***********************************************************************************************/

void MailKount::help()
{
	KMessageBox::information(0, i18n("This is a help box"));
}

/***********************************************************************************************/

void MailKount::resizeEvent(QResizeEvent *e)
{
}

/***********************************************************************************************/

extern "C"
{
	KPanelApplet* init(QWidget *parent, const QString& configFile)
	{
		KGlobal::locale()->insertCatalogue("mailkount");
		return new MailKount(configFile, KPanelApplet::Normal,
			KPanelApplet::About | KPanelApplet::Preferences,
			parent, "Mailkount");
	}
}
